#include <stdlib.h>

#include <firestorm.h>
#include <packet.h>
#include <plugin.h>
#include <generator.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>

PLUGIN_STD_DEFS();

void arp_decode(struct packet *);
void rarp_decode(struct packet *);

struct proto rarp_p=init_proto("rarp", rarp_decode, NULL);
struct proto_req rarp_r[]={
	proto_request("ethernet", __constant_htons(0x0835)),
	proto_request("sll", __constant_htons(0x0835)),
	proto_request("linux", __constant_htons(0x0835)),
	null_request()
};

struct proto arp_p=init_proto("arp", arp_decode, NULL);
struct proto_req arp_r[]={
	/* IP ARP */
	proto_request("ethernet", __constant_htons(0x0806)),
	proto_request("sll", __constant_htons(0x0806)),
	proto_request("linux", __constant_htons(0x0806)),

	/* AppleTalk ARP */
	proto_request("ethernet", __constant_htons(0x80f3)),
	proto_request("sll", __constant_htons(0x80f3)),
	proto_request("linux", __constant_htons(0x80f3)),
	
	null_request()
};

void arp_decode(struct packet *p)
{
	struct layer *l=&p->layer[p->llen];

	if ( l->h.raw+sizeof(struct pkt_arphdr) > p->end ) {
		return;
	}

	p->llen++;
}

void rarp_decode(struct packet *p)
{
	struct layer *l=&p->layer[p->llen];

	if ( l->h.raw+sizeof(struct pkt_arphdr) > p->end ) {
		return;
	}

	p->llen++;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	int ok=0;
	proc_decode_add decode_add;

	plugin_check(in, out);
	
	PLUGIN_ID("decode.arp", "ARP/RARP");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=decode_add(&arp_p, arp_r);
	ok+=decode_add(&rarp_p, rarp_r);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
